#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <libgyim.h>

/* Kiszamolja a dal sorainak szamat. Ilyen alaku fajlt olvas be:
sorok =
ambitus =
szotag =
kadencia =
x =
...
*/

static void egydal(FILE *fp,char *szocikk);
static void sorveg(char *sor);		// levagja \n-et es a helykozoket a sor vegerol
static int  spaceszam(char *sor);	// a szavak szama a stringben
static int  hangkonv(char *hang);	// ambitusbeli hang konverzioja
static char *hangtavki(int tav);	// hangtav kiirasa felhangban megadott parameterrel

static const char space[] = " ", be[] = "♭", kereszt[] = "♯";
static const int arab[] = { 0, 2, 4, 5, 7, 9, 10 };	// felhangok es zenei hangkozok
//                          1  2  3  4  5  6  7

#define	SORHOSSZ	300



int main(int argc, char *argv[])
{
FILE *fp;

if(argc < 3)
	{
	fprintf(stderr,"Hivas: %s metafajl szocikkcim\n",argv[0]);
	exit(1);
	}
fp = filenyit(argv[1],"r");
egydal(fp,argv[2]);
return 0;
}


static void egydal(FILE *fp,char *szocikk)
{
enum sortip { SOROK, SZOTAG, KADENCIA, AMBITUS };
static const char *mezonev[] = { "sorok", "szótag", "kadencia", "ambitus" }, tol[] = "–", kadstr[] = " ()";
char mezotomb[4][SORHOSSZ],sor[SORHOSSZ],*mut,dalmin[SORHOSSZ],dalmax[SORHOSSZ];
int i,i1,i2,hossz,min,max;

for(i=0; i < sizeof(mezotomb)/SORHOSSZ; i++)
	mezotomb[i][0] = 0;
while(fgets(sor,sizeof(sor),fp) != NULL )
	{		// egy dal metaadatainak beolvasasa
	sorveg(sor);
	hossz  = strcspn(sor," =");
	for(i=0; i < sizeof(mezonev)/sizeof(char *); i++)
		if(memcmp(mezonev[i],sor,hossz) == 0)
			{
			strcpy(mezotomb[i],sor + hossz + strspn(sor+hossz," ="));
			break;
			}
	}
for(i=max=0,min=100; i < sizeof(mezotomb)/SORHOSSZ; i++)
	{		// dalsor kiszamitasa
	if( (hossz = spaceszam(mezotomb[i]) ) == 0)
		continue;
	if(hossz && i == KADENCIA)
		hossz++;
	if(min > hossz)
		min = hossz;
	if(max < hossz)
		max = hossz;
	}
if(max == 0)
	return;	// nem volt olyan sor, amibol dalsor kiszamithato. Nincs tobb dolgunk.
if(min != max)		// dalsor kiirasa
	{
	fprintf(stderr,"%s: A sorok szama hibas: %d != %d\n",szocikk,min,max);
	return;
	}
else	printf("dalsor = %d\n",min);

			/* Ambitus */
if(mezotomb[AMBITUS][0])
	{
	min = 100; max = -100;	// a legkisebb es legnagyobb sor-ambitus a dalban
	dalmin[0] = dalmax[0] = 0;
	for(i=1,mut = strtok(mezotomb[AMBITUS],space); mut != NULL; mut = strtok(NULL,space),i++)
		{	// ambitus konverzioja.
		char *mut1,*mut2;
	
		mut1 = strstr(mut,tol);
		if(mut1 == NULL)
			{
			fprintf(stderr,"%s: Az ambitusban kell legyen %s\n",szocikk,tol);
			return;
			}
		else	*mut1 = 0;
		mut2 = mut1 + sizeof(tol)-1;
		while(--mut1 >= mut && *mut1 == ' ')
			*mut1 = 0;
		mut2 += strspn(mut2,space);	// a felso hatar szovegesen
		i1 = hangkonv(mut);		// az also hatar szam formaban
		i2 = hangkonv(mut2);		// a felso hatar szam formaban
		if(i1 > i2)
			{
			fprintf(stderr,"%s: Az also ambitus legyen kisebb a felsonel\n",szocikk);
			return;
			}
		//printf("%s-%s, %d %d %d\n",mut,mut2,i1,i2,i2-i1);
		if(min > i1)
			{
			min = i1;
			strcpy(dalmin,mut);
			}
		if(max < i2)
			{
			max = i2;
			strcpy(dalmax,mut2);
			}
	//		Egesz sor	Also amb.	Felso amb.
	//	Szam	amb{i}sor
	//	Szoveg	               	amb{i}s1	amb{i}s2
		printf("amb%ds1 = %s\namb%ds2 = %s\n",i,mut,i,mut2);	// az also es felso hang szovegesen
		printf("amb%dsor = %s\n",i,hangtavki(i2-i1));		// a teljes sor szam formaban
		}
	printf("ambmin = %s\nambmax = %s\nambdal = %s\n",dalmin,dalmax,hangtavki(max-min));
	}

			/* Szotag */
for(i=1,mut = strtok(mezotomb[SZOTAG],space); mut != NULL; mut = strtok(NULL,space),i++)
	printf("szot%d = %s\n",i,mut);

			/* Kadencia */
for(i=1,mut = strtok(mezotomb[KADENCIA],kadstr); mut != NULL; mut = strtok(NULL,kadstr),i++)
	printf("kad%d = %s\n",i,mut);
}


static void sorveg(char *sor)
{
char *mut;

for(mut=sor+strlen(sor)-1; mut >= sor; )
	if(*mut == '\n' || *mut == ' ')
		*mut-- = 0;
	else	break;
}



static int  spaceszam(char *sor)
{	// a szavak szama a stringben
char *mut = sor;
int ret = 0;

while(*mut)
	{
	ret++;
	mut += strcspn(mut,space);
	mut += strspn(mut,space);
	}
return ret;
}



static int  hangkonv(char *hang)	// ambitusbeli hang konverzioja. 1 -> 0, 
{
static const char romai[][5] = { "I", "II", "III", "IV", "V", "VI", "VII" };
int i,ret = 0;

if(memcmp(hang,be,strlen(be)) == 0)
	{
	ret = -1;
	hang += sizeof(be)-1;
	}
if(memcmp(hang,kereszt,strlen(kereszt)) == 0)
	{
	ret = 1;
	hang += sizeof(kereszt)-1;
	}
if(isupper(*hang))
	{
	for(i=0; i < sizeof(romai)/sizeof(romai[0]); i++)
		if(strcmp(romai[i],hang) == 0)
			break;
	return arab[i] - 12 + ret;
	}
if(isdigit(*hang))
	{
	int munka;

	sscanf(hang,"%d",&munka);
	munka = munka > 0? munka-1: 0;
	return ret + (munka/7)*12 + arab[munka%7];
	}
return ret;
}


#if	0
int main(int argc,char *argv[])
{
if(argc < 2)
	{
	fprintf(stderr,"Hivas: dalsor ambitus\n");
	return 1;
	}
printf("%s    %d\n",argv[1],hangkonv(argv[1]));
return 0;
}
#endif


static char *hangtavki(int tav)		// hangtav kiirasa felhangban megadott parameterrel
{
static char ret[20];
char *mut;
int i,t;

t = tav%12;
if(t == 11)
	{
	strcpy(ret,kereszt);
	mut = ret + strlen(ret);
	i = 6;
	}
else	{
	for(i=0; i < 7; i++)
		if(t <= arab[i])
			break;
	if(t < arab[i])
		{
		strcpy(ret,be);
		mut = ret + strlen(ret);
		}
	else	mut = ret;
	}
t = 8*(tav/12);
if(!t) i++;
sprintf(mut,"%d",t+i);
return ret;
}


#if	0 

int main(int argc,char *argv[])
{
int tmp;

if(argc < 2 || sscanf(argv[1],"%d",&tmp) < 1)
	{
	fputs("Hivas: dalsor hangtav\n",stderr);
	return 1;
	}
else	printf("Hangtav = %d\n",tmp);
puts(hangtavki(tmp));
return 0;
}

#endif
